@@ -5,7 +5,7 @@
 
  This is an open-source copyright as follows:
  Copyright (c) 2004-2008 BioImageXD Development Team
- Copyright (C) 2012 Luis Pedro Coelho
+ Copyright (C) 2012-2020 Luis Pedro Coelho
 
  All rights reserved.
 
@@ -154,7 +154,7 @@ class LSMReader {
         ~LSMReader();
 
         void PrintSelf(std::ostream& os, const char* indent="");
-        std::auto_ptr<Image> read(ImageFactory* factory, const options_map&);
+        std::unique_ptr<Image> read(ImageFactory* factory, const options_map&);
         void readHeader();
 
         int GetChannelColorComponent(int,int);
@@ -167,10 +167,10 @@ class LSMReader {
 
     private:
 
+        static int FindChannelNameStart(const byte *, int);
+        static int ReadChannelName(const byte *, int, byte *);
+
         unsigned long ReadImageDirectory(byte_source *,unsigned long);
-        void SetChannelName(const char *,int);
-        int FindChannelNameStart(const char *, int);
-        int ReadChannelName(const char *, int, char *);
         int ReadChannelDataTypes(byte_source*, unsigned long);
         int ReadChannelColorsAndNames(byte_source *,unsigned long);
         int ReadTimeStampInformation(byte_source *,unsigned long);
@@ -232,45 +232,51 @@ class LSMReader {
 
 };
 
-int ReadFile(byte_source* s, unsigned long *pos,int size,void *buf,bool swap=false)
+int ReadFile(byte_source* s, unsigned long *pos, int size, void *buf, bool swap=false)
 {
-  s->seek_absolute(*pos);
-  const unsigned ret = s->read(reinterpret_cast<byte*>(buf), size);
+    s->seek_absolute(*pos);
+    const unsigned ret = s->read(reinterpret_cast<byte*>(buf), size);
 #ifdef VTK_WORDS_BIGENDIAN
   if(swap) {
     vtkByteSwap::SwapLERange(buf,size);
   }
 #endif
-  *pos += ret;
-  return ret;
+    *pos += ret;
+    return ret;
 }
 
 
-int ReadData(byte_source* s, unsigned long *pos,int size,char *buf) {
-  return ReadFile(s,pos,size,buf,1);
-}
-
 std::string read_str(byte_source* s, unsigned long* pos, const unsigned long len) {
     char* buf = new char[len];
-    ReadData(s, pos, len, buf);
+    ReadFile(s, pos, len, buf, 1);
     std::string res(buf, len);
     delete [] buf;
     return res;
 }
 
-unsigned char CharPointerToUnsignedChar(char *buf)
-{
-  return *((unsigned char*)(buf));
-}
 
 int CharPointerToInt(char *buf)
 {
   return *((int*)(buf));
 }
 
-unsigned int CharPointerToUnsignedInt(char *buf)
-{
-  return *((unsigned int*)(buf));
+
+uint32_t parse_uint32_t(const byte *buf) {
+#ifdef VTK_WORDS_BIGENDIAN
+    char buf2[4];
+    ::memcpy(buf2, buf, 4);
+    vtkByteSwap::Swap4LE(buf2);
+    return *reinterpret_cast<const uint32_t*>(buf2);
+#else
+    return *reinterpret_cast<const uint32_t*>(buf);
+#endif
+}
+
+uint32_t parse_uint32_t(const std::vector<byte> &buf) {
+    if (buf.size() < 4) {
+        throw CannotReadError("Malformed LSM file: expected 4 Bytes, cannot parse uint32_t");
+    }
+    return parse_uint32_t(buf.data());
 }
 
 short CharPointerToShort(char *buf)
@@ -278,9 +284,22 @@ short CharPointerToShort(char *buf)
   return *((short*)(buf));
 }
 
-unsigned short CharPointerToUnsignedShort(char *buf)
-{
-  return *((unsigned short*)(buf));
+
+uint16_t parse_uint16_t(const byte *buf) {
+#ifdef VTK_WORDS_BIGENDIAN
+    char buf2[2];
+    buf2[0] = buf[1];
+    buf2[1] = buf[0];
+    return *reinterpret_cast<const uint16_t*>(buf2);
+#else
+    return *reinterpret_cast<const uint16_t*>(buf);
+#endif
+}
+uint16_t parse_uint16_t(const std::vector<byte>& buf) {
+    if (buf.size() < 2) {
+        throw CannotReadError("Failed to read short (size(vec) < 2)");
+    }
+    return parse_uint16_t(buf.data());
 }
 
 double CharPointerToDouble(char *buf)
@@ -298,14 +317,10 @@ int ReadInt(byte_source* s, unsigned long *pos)
   return CharPointerToInt(buff);
 }
 
-unsigned int ReadUnsignedInt(byte_source* s, unsigned long *pos)
-{
-  char buff[4];
-  ReadFile(s,pos,4,buff);
-#ifdef VTK_WORDS_BIGENDIAN
-  vtkByteSwap::Swap4LE((unsigned int*)buff);
-#endif
-  return CharPointerToUnsignedInt(buff);
+unsigned int ReadUnsignedInt(byte_source* s, unsigned long *pos) {
+    byte buff[4];
+    ReadFile(s, pos, 4, buff);
+    return parse_uint32_t(buff);
 }
 
 short ReadShort(byte_source* s, unsigned long *pos)
@@ -318,14 +333,10 @@ short ReadShort(byte_source* s, unsigned long *pos)
   return CharPointerToShort(buff);
 }
 
-unsigned short ReadUnsignedShort(byte_source* s, unsigned long *pos)
-{
-  char buff[2];
-  ReadFile(s,pos,2,buff);
-#ifdef VTK_WORDS_BIGENDIAN
-  vtkByteSwap::Swap2LE((unsigned short*)buff);
-#endif
-  return CharPointerToUnsignedShort(buff);
+uint16_t ReadUnsignedShort(byte_source* s, unsigned long *pos) {
+    byte buff[2];
+    ReadFile(s,pos,2,buff);
+    return parse_uint16_t(buff);
 }
 
 double ReadDouble(byte_source* s, unsigned long *pos)
@@ -436,23 +447,15 @@ std::string LSMReader::GetChannelName(int chNum)
     return this->channel_names_[chNum];
 }
 
-void LSMReader::SetChannelName(const char * name, const int chNum)
-{
-    const int n_channels = this->dimensions_[4];
-    if(!name || chNum > n_channels) return;
-    this->channel_names_.resize(n_channels);
-    this->channel_names_[chNum] = std::string(name);
-}
 
-int LSMReader::FindChannelNameStart(const char *buf, const int length) {
+int LSMReader::FindChannelNameStart(const byte *buf, const int length) {
     for (int i = 0; i < length; ++i) {
-        char ch = buf[i];
-        if (ch > 32) return i;
+        if (buf[i] > 32) return i;
     }
     return length;
 }
 
-int LSMReader::ReadChannelName(const char *nameBuff, const int length, char *buffer) {
+int LSMReader::ReadChannelName(const byte *nameBuff, const int length, byte *buffer) {
     for (int i = 0; i < length; ++i) {
         buffer[i] = nameBuff[i];
         if (!buffer[i]) return i;
@@ -472,67 +475,64 @@ int LSMReader::ReadChannelDataTypes(byte_source* s, unsigned long start)
     return 0;
 }
 
-int LSMReader::ReadChannelColorsAndNames(byte_source* s, unsigned long start)
-{
-    int colNum,nameNum,sizeOfStructure,sizeOfNames,nameLength, nameSkip;
-    unsigned long colorOffset,nameOffset,pos;
-    char *nameBuff,*name,*tempBuff;
-    unsigned char component;
-
-    pos = start;
+int LSMReader::ReadChannelColorsAndNames(byte_source* s, const unsigned long start) {
+    unsigned long pos = start;
     // Read size of structure
-    sizeOfStructure = ReadInt(s,&pos);
+    const int sizeOfStructure = ReadInt(s, &pos);
     // Read number of colors
-    colNum = ReadInt(s,&pos);
+    const int n_cols = ReadInt(s, &pos);
     // Read number of names
-    nameNum = ReadInt(s,&pos);
-    sizeOfNames = sizeOfStructure - ( (10*4) + (colNum*4) );
+    const int n_names = ReadInt(s, &pos);
+    const int sizeOfNames = sizeOfStructure - ( (10*4) + (n_cols*4) );
 
-    nameBuff = new char[sizeOfNames+1];
-    name = new char[sizeOfNames+1];
-
-    if(colNum != this->dimensions_[4]) {
-        // not great
+    if(n_cols != this->dimensions_[4]) {
+        throw CannotReadError("LSM file seems malformed (n_cols != dimensions_[4])");
     }
-    if(nameNum != this->dimensions_[4]) {
-        // not great
+    if(n_names != this->dimensions_[4]) {
+        throw CannotReadError("LSM file seems malformed (n_names != dimensions_[4])");
     }
 
     // Read offset to color info
-    colorOffset = ReadInt(s,&pos) + start;
+    unsigned long colorOffset = ReadInt(s, &pos) + start;
     // Read offset to name info
-    nameOffset = ReadInt(s,&pos) + start;
+    unsigned long nameOffset = ReadInt(s, &pos) + start;
 
-    this->channel_colors_.resize( 3 *  (colNum+1));
+    this->channel_colors_.resize( 3 *  (n_cols+1));
 
   // Read the colors
     for(int j = 0; j < this->dimensions_[4]; ++j) {
         char colorBuff[5];
-        ReadFile(s,&colorOffset,4,colorBuff,1);
+        ReadFile(s, &colorOffset, 4, colorBuff, 1);
         for(int i=0;i<3;i++) {
-            component = CharPointerToUnsignedChar(colorBuff+i);
-            this->channel_colors_[i + 3*j] = component;
+            this->channel_colors_[i + 3*j] = static_cast<unsigned char>(colorBuff[i]);
         }
     }
 
-    ReadFile(s,&nameOffset,sizeOfNames,nameBuff,1);
 
-    nameLength = nameSkip = 0;
-    tempBuff = nameBuff;
+    std::vector<byte> nameBuff;
+    nameBuff.resize(sizeOfNames + 1);
+    std::vector<byte> name;
+    name.resize(sizeOfNames + 1);
+
+    ReadFile(s, &nameOffset, sizeOfNames, nameBuff.data(), 1);
+
+    this->channel_names_.resize(this->dimensions_[4]);
+    int nameStart = 0;
     for(int i = 0; i < this->dimensions_[4]; i++) {
-        nameSkip = this->FindChannelNameStart(tempBuff,sizeOfNames-nameSkip);
-        nameLength = this->ReadChannelName(tempBuff+nameSkip,sizeOfNames-nameSkip,name);
+        nameStart += FindChannelNameStart(nameBuff.data() + nameStart,
+                                            sizeOfNames-nameStart);
+        if (nameStart >= sizeOfNames) {
+            throw CannotReadError("LSM file malformed");
+        }
+        const int nameLength = ReadChannelName(nameBuff.data()+nameStart, sizeOfNames-nameStart, name.data());
+        nameStart += nameLength;
 
-        tempBuff += nameSkip + nameLength;
-        this->SetChannelName(name,i);
+        this->channel_names_[i] = std::string(reinterpret_cast<const char*>(name.data()));
     }
-    delete [] nameBuff;
-    delete [] name;
     return 0;
 }
 
-int LSMReader::ReadTimeStampInformation(byte_source* s, unsigned long offset)
-{
+int LSMReader::ReadTimeStampInformation(byte_source* s, unsigned long offset) {
     // position is 0 for non-timeseries files!
     if( offset == 0 ) return 0;
 
@@ -551,9 +551,7 @@ int LSMReader::ReadTimeStampInformation(byte_source* s, unsigned long offset)
  *
  *
  */
-int LSMReader::ReadLSMSpecificInfo(byte_source* s, unsigned long pos)
-{
-  unsigned long offset;
+int LSMReader::ReadLSMSpecificInfo(byte_source* s, unsigned long pos) {
 
   pos += 2 * 4; // skip over the start of the LSMInfo
                 // first 4 byte entry if magic number
@@ -645,7 +643,7 @@ int LSMReader::ReadLSMSpecificInfo(byte_source* s, unsigned long pos)
   // SKip Zeiss Vision KS-3D speific data
   pos +=  4;
   // Read timestamp information
-  offset = ReadUnsignedInt(s,&pos);
+  unsigned long offset = ReadUnsignedInt(s, &pos);
   this->ReadTimeStampInformation(s,offset);
 
   return 1;
@@ -765,156 +763,111 @@ int LSMReader::ReadScanInformation(byte_source* s,  unsigned long pos)
     } while (subblocksOpen);
     return 0;
 }
-int LSMReader::AnalyzeTag(byte_source* s, unsigned long startPos)
-{
-  int value, dataSize;
-  char tempValue[4],tempValue2[4];
-  char *actualValue = NULL;
-  const unsigned short tag = ReadUnsignedShort(s,&startPos);
-  const unsigned short type = ReadUnsignedShort(s,&startPos);
-  const unsigned short length = ReadUnsignedInt(s,&startPos);
 
-  ReadFile(s,&startPos,4,tempValue);
+int LSMReader::AnalyzeTag(byte_source* s, unsigned long startPos) {
+    std::vector<byte> valueData;
+    const unsigned short tag = ReadUnsignedShort(s, &startPos);
+    const unsigned short type = ReadUnsignedShort(s, &startPos);
+    const unsigned short length = ReadUnsignedInt(s, &startPos);
 
-  for(int i=0;i<4;i++)tempValue2[i]=tempValue[i];
-#ifdef VTK_WORDS_BIGENDIAN
-  vtkByteSwap::Swap4LE((unsigned int*)tempValue2);
-#endif
-  value = CharPointerToUnsignedInt(tempValue2);
+    valueData.resize(4);
+    ReadFile(s, &startPos, 4, valueData.data());
 
-  // if there is more than 4 bytes in value,
-  // value is an offset to the actual data
-  dataSize = TIFF_BYTES(type);
-  const unsigned long readSize = dataSize*length;
-  if(readSize > 4 && tag != TIF_CZ_LSMINFO)
-  {
-    actualValue = new char[readSize];
-    startPos = value;
-   if(tag == TIF_STRIPOFFSETS ||tag == TIF_STRIPBYTECOUNTS) {
-        if( !ReadFile(s,&startPos,readSize,actualValue) ) {
-            throw CannotReadError("Failed to get strip offsets\n");
-        }
-    }
-  }
-  else
-  {
-      actualValue = new char[4];
-      for(int o=0;o<4;o++)actualValue[o] = tempValue[o];
-  }
-  switch(tag)
-  {
-    case TIF_NEWSUBFILETYPE:
-      this->NewSubFileType = value;
-      break;
+    const int value = parse_uint32_t(valueData);
 
-    case TIF_IMAGEWIDTH:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap4LE((unsigned int*)actualValue);
-#endif
-      //this->dimensions_[0] = this->CharPointerToUnsignedInt(actualValue);
-      //this->dimensions_[0] = value;
-      break;
-
-    case TIF_IMAGELENGTH:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap4LE((unsigned int*)actualValue);
-      //this->dimensions_[1] = this->CharPointerToUnsignedInt(actualValue);
-#endif
-      //this->dimensions_[1] = value;
-      break;
-
-    case TIF_BITSPERSAMPLE:
-#ifdef VTK_WORDS_BIGENDIAN
-        vtkByteSwap::Swap2LE((unsigned short*)actualValue);
-#endif
-        this->bits_per_sample_.resize(length);
-        unsigned short bits_per_sample_;
-        for(int i=0;i<length;i++)
-        {
-           bits_per_sample_ = CharPointerToUnsignedShort(actualValue + TIFF_BYTES(TIFF_SHORT)*i);
-           this->bits_per_sample_[i] = bits_per_sample_;
-        }
-        break;
-
-    case TIF_COMPRESSION:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap2LE((unsigned short*)actualValue);
-#endif
-      this->compression_ = CharPointerToUnsignedShort(actualValue);
-      break;
-
-    case TIF_PHOTOMETRICINTERPRETATION:
-#ifdef VTK_WORDS_BIGENDIAN
-        vtkByteSwap::Swap2LE((unsigned short*)actualValue);
-#endif
-        this->PhotometricInterpretation = CharPointerToUnsignedShort(actualValue);
-        break;
-
-    case TIF_STRIPOFFSETS:
-        this->strip_offset_.resize(length);
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap4LERange((unsigned int*)actualValue,length);
-#endif
-        if(length>1) {
-            for(int i=0;i<length;++i)
-            {
-                unsigned int* offsets = (unsigned int*)actualValue;
-                this->strip_offset_[i] = offsets[i];
+    // if there is more than 4 bytes in value,
+    // value is an offset to the actual data
+    const int dataSize = TIFF_BYTES(type);
+    const unsigned long readSize = dataSize*length;
+    if(readSize > 4 && tag != TIF_CZ_LSMINFO) {
+        valueData.resize(readSize);
+        startPos = value;
+        if(tag == TIF_STRIPOFFSETS ||tag == TIF_STRIPBYTECOUNTS) {
+            if( !ReadFile(s, &startPos, readSize, valueData.data()) ) {
+                throw CannotReadError("Failed to get strip offsets\n");
             }
-        } else {
-            this->strip_offset_[0] = value;
         }
-        break;
+    }
+    switch(tag) {
+        case TIF_NEWSUBFILETYPE:
+            this->NewSubFileType = value;
+            break;
 
-    case TIF_SAMPLESPERPIXEL:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap4LE((unsigned int*)actualValue);
-#endif
-      this->sample_per_pixel_ = CharPointerToUnsignedInt(actualValue);
-      break;
+        case TIF_IMAGEWIDTH:
+            //this->dimensions_[0] = parse_uint32_t(valueData);
+            //this->dimensions_[0] = value;
+            break;
 
-    case TIF_STRIPBYTECOUNTS:
-#ifdef VTK_WORDS_BIGENDIAN
-        vtkByteSwap::Swap4LERange((unsigned int*)actualValue,length);
-#endif
-        this->strip_byte_count_.resize(length);
-        if (length > 1) {
-            for(int i=0; i<length; ++i) {
-                this->strip_byte_count_[i] = CharPointerToUnsignedInt(actualValue + TIFF_BYTES(TIFF_LONG)*i);
+        case TIF_IMAGELENGTH:
+            //this->dimensions_[1] = parse_uint32_t(valueData);
+            //this->dimensions_[1] = value;
+            break;
+
+        case TIF_BITSPERSAMPLE:
+            if (valueData.size() < TIFF_BYTES(TIFF_SHORT) * length) {
+                throw CannotReadError("LSM file is malformed (TIF_BITSPERSAMPLE field is too short)");
             }
-        } else {
-            this->strip_byte_count_[0] = value;
-        }
-        break;
-    case TIF_PLANARCONFIGURATION:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap2LE((unsigned short*)actualValue);
-#endif
-      this->PlanarConfiguration = CharPointerToUnsignedShort(actualValue);
-      break;
-    case TIF_PREDICTOR:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap2LE((unsigned short*)actualValue);
-#endif
-      this->Predictor = CharPointerToUnsignedShort(actualValue);
-      break;
-    case TIF_COLORMAP:
-#ifdef VTK_WORDS_BIGENDIAN
-      vtkByteSwap::Swap4LE((unsigned int*)actualValue);
-#endif
-      //this->ColorMapOffset = CharPointerToUnsignedInt(actualValue);
-      break;
-    case TIF_CZ_LSMINFO:
+            this->bits_per_sample_.resize(length);
+            for(int i=0;i<length;i++) {
+                this->bits_per_sample_[i] = parse_uint16_t(valueData.data() + TIFF_BYTES(TIFF_SHORT)*i);
+            }
+            break;
 
-      this->LSMSpecificInfoOffset = value;
-      break;
+        case TIF_COMPRESSION:
+            this->compression_ = parse_uint16_t(valueData);
+            break;
+
+        case TIF_PHOTOMETRICINTERPRETATION:
+            this->PhotometricInterpretation = parse_uint16_t(valueData);
+            break;
+
+        case TIF_STRIPOFFSETS:
+            this->strip_offset_.resize(length);
+            if(length>1) {
+                if (length * sizeof(uint32_t) > valueData.size()) {
+                    throw CannotReadError("LSM file is malformed (TIF_STRIPOFFSETS field is too short)");
+                }
+                for(int i=0;i<length;++i) {
+                    this->strip_offset_[i] = parse_uint32_t(valueData.data() + sizeof(uint32_t) * i);
+                }
+            } else {
+                this->strip_offset_[0] = value;
+            }
+            break;
+
+        case TIF_SAMPLESPERPIXEL:
+            this->sample_per_pixel_ = parse_uint32_t(valueData);
+            break;
+
+        case TIF_STRIPBYTECOUNTS:
+            this->strip_byte_count_.resize(length);
+            if (length > 1) {
+                for(int i=0; i<length; ++i) {
+                    if (valueData.size() < TIFF_BYTES(TIFF_LONG) * i + 4) {
+                        throw CannotReadError();
+                    }
+                    this->strip_byte_count_[i] = parse_uint32_t(valueData.data() + TIFF_BYTES(TIFF_LONG)*i);
+                }
+            } else {
+                this->strip_byte_count_[0] = value;
+            }
+            break;
+        case TIF_PLANARCONFIGURATION:
+            this->PlanarConfiguration = parse_uint16_t(valueData);
+            break;
+        case TIF_PREDICTOR:
+            this->Predictor = parse_uint16_t(valueData);
+            break;
+        case TIF_COLORMAP:
+            //this->ColorMapOffset = parse_uint32_t(valueData);
+            break;
+        case TIF_CZ_LSMINFO:
+
+            this->LSMSpecificInfoOffset = value;
+            break;
     }
 
-  if(actualValue)
-    {
-    delete [] actualValue;
-    }
-  return 0;
+    return 0;
 }
 
 
@@ -1017,8 +970,8 @@ void LSMReader::DecodeLZWCompression(unsigned char* buffer, int size, int bytes)
     std::vector<unsigned char> decoded = lzw_decode(buffer, size);
     unsigned char* outbufp = &decoded[0];
 
-    int width = this->dimensions_[0];
-    int lines = size / (width*bytes);
+    const int width = this->dimensions_[0];
+    const int lines = size / (width*bytes);
 
     for(int line = 0; line < lines; line++) {
         if(this->Predictor == 2) {
@@ -1136,12 +1089,12 @@ void LSMReader::PrintSelf(std::ostream& os, const char* indent)
     }
 }
 
-std::auto_ptr<Image> LSMReader::read(ImageFactory* factory, const options_map&) {
+std::unique_ptr<Image> LSMReader::read(ImageFactory* factory, const options_map&) {
     this->readHeader();
 
     const int dataType = this->GetDataTypeForChannel(0); // This could vary by channel!
 
-    std::auto_ptr<Image> output = factory->create(
+    std::unique_ptr<Image> output = factory->create(
                             BYTES_BY_DATA_TYPE(dataType)*8,
                             this->dimensions_[2],
                             this->dimensions_[3],
@@ -1178,7 +1131,7 @@ std::auto_ptr<Image> LSMReader::read(ImageFactory* factory, const options_map&)
 
 } // namespace
 
-std::auto_ptr<Image> LSMFormat::read(byte_source* s, ImageFactory* factory, const options_map& opts) {
+std::unique_ptr<Image> LSMFormat::read(byte_source* s, ImageFactory* factory, const options_map& opts) {
     LSMReader reader(s);
     return reader.read(factory, opts);
 }
