@@ -1,4 +1,4 @@
-// Copyright 2012 Luis Pedro Coelho <luis@luispedro.org>
+// Copyright 2012-2019 Luis Pedro Coelho <luis@luispedro.org>
 // License: MIT (see COPYING.MIT file)
 
 #ifndef LPC_WEBP_H_INCLUDE_GUARD_
@@ -8,7 +8,7 @@ class WebPFormat : public ImageFormat {
         bool can_read() const { return true; }
         bool can_write() const { return false; }
 
-        std::auto_ptr<Image> read(byte_source* src, ImageFactory* factory, const options_map&);
+        std::unique_ptr<Image> read(byte_source* src, ImageFactory* factory, const options_map&);
 };
 
 #endif // LPC_WEBP_H_INCLUDE_GUARD_
